@@ -19,6 +19,16 @@ try:
 except Exception:
     HAS_DATEUTIL = False
 
+PY_37 = sys.version_info[:2] == (3, 7)
+
+# We have to write the condition like this for mypy to recognise it
+if sys.version_info >= (3, 8):
+    from typing import get_origin as typing_get_origin
+    from typing import get_args as typing_get_args
+else:
+    from typing_extensions import get_origin as typing_get_origin
+    from typing_extensions import get_args as typing_get_args
+
 
 _FROM = 1
 _TO = 2
@@ -150,8 +160,8 @@ def _from_dict_source(cls):
     for name, field_type in typing.get_type_hints(cls).items():
 
         # pop off the top layer of optional, since we are using o.get
-        if typing.get_origin(field_type) == typing.Union:
-            field_type = typing.get_args(field_type)[0]
+        if typing_get_origin(field_type) == typing.Union:
+            field_type = typing_get_args(field_type)[0]
 
         field = fields_by_name[name]
 
@@ -235,10 +245,10 @@ def _to_dict_source(cls):
         if transform('x') != 'x':
             # since we have an is not none check, elide the first level
             # of optional
-            if (typing.get_origin(field_type) == typing.Union
+            if (typing_get_origin(field_type) == typing.Union
                     # This is a bit yuk. Premature optimization 🙄
                     and transform('x') == expr_builder_to(field_type)('x')):
-                transform = expr_builder_to(typing.get_args(field_type)[0])
+                transform = expr_builder_to(typing_get_args(field_type)[0])
             lines.append(f'    value = {access}')
             lines.append(f'    if value is not None:')  # noqa: F541
             lines.append(f'        value = ' + transform('value'))  # noqa: E501,F541
@@ -302,19 +312,22 @@ def expr_builder(t: type, depth=0, direction=_FROM):
 
     # Container types
 
-    origin = typing.get_origin(t)
+    origin = typing_get_origin(t)
 
     if origin == typing.Union:
-        type_arg = typing.get_args(t)[0]
+        type_arg = typing_get_args(t)[0]
         inner = expr_builder(type_arg, depth + 1, direction)
 
         def f(expr):
             t0 = f'__{depth}'
+            if PY_37:
+                # Lazy solution for py37'ers
+                return f'{inner(expr)} if ({expr}) is not None else None'
             return f'{inner(t0)} if ({t0}:=({expr})) is not None else None'
 
         return f
-    elif origin == tuple and typing.get_args(t):
-        type_args = typing.get_args(t)
+    elif origin == tuple and typing_get_args(t):
+        type_args = typing_get_args(t)
         # Tuple[A, ...] means an any-length tuple of all As
         if type_args[1:] == (Ellipsis,):
             inner = expr_builder(type_args[0], depth + 1, direction)
@@ -336,17 +349,21 @@ def expr_builder(t: type, depth=0, direction=_FROM):
                 parts = ['(']
                 for i, inner in enumerate(inners):
                     xx = f'{t0}[{i}]'
+                    if PY_37:
+                        xx = f'({expr})[{i}]'
                     parts.append(f'{inner(xx)},')
                 parts.append(')')
                 e2 = ''.join(parts)
+                if PY_37:
+                    return e2
                 # e1 is just for evaluating expr no more than once
                 # e2 is the actual result
                 return f'({e1},{e2})[1]'
             return f
     elif (issubclass_safe(origin, abc.Sequence)
           and issubclass_safe(list, origin)
-          and typing.get_args(t)):
-        type_arg = typing.get_args(t)[0]
+          and typing_get_args(t)):
+        type_arg = typing_get_args(t)[0]
         inner = expr_builder(type_arg, depth + 1, direction)
 
         def f(expr):
@@ -355,8 +372,8 @@ def expr_builder(t: type, depth=0, direction=_FROM):
         return f
     elif (issubclass_safe(origin, abc.Mapping)
           and issubclass_safe(dict, origin)
-          and typing.get_args(t)):
-        key_type, value_type = typing.get_args(t)
+          and typing_get_args(t)):
+        key_type, value_type = typing_get_args(t)
 
         # TODO: Enums, dates and decimals should be trivial to add here
         if key_type not in (str, int, float, bool, UUID):
@@ -372,8 +389,9 @@ def expr_builder(t: type, depth=0, direction=_FROM):
             elif key_type in (int, float):
                 key_func = lambda k: f'{key_type.__name__}({k})'
             elif key_type is bool:
-                t0 = f'__{depth}'
-                key_func = lambda k: f'(({t0}:=({k})) is True or {k} == "true")'
+                # The double evaluation of k is fine since k always
+                # an iteration variable in f below
+                key_func = lambda k: f'({k} is True or {k} == "true")'
 
         def f(expr):
             k0 = f'__k{depth}'
@@ -425,6 +443,11 @@ def expr_builder(t: type, depth=0, direction=_FROM):
                 if HAS_DATEUTIL and issubclass_safe(datetime, t):
                     return f'dateutil.parser.isoparse({expr})'
                 else:
+                    if PY_37:
+                        return (f'{t.__name__}.fromisoformat('
+                                f'({expr})[:-1]+"+00:00" if ({expr})[-1]=="Z" '
+                                f'else ({expr})'
+                                ')')
                     return (f'{t.__name__}.fromisoformat('
                             f'{t0}[:-1]+"+00:00" if ({t0}:={expr})[-1]=="Z" '
                             f'else {t0}'
@@ -459,18 +482,18 @@ def referenced_types(cls):
     # If we support tuples or unions properly, this needsto return
     # multiple types
     def extract_types(t):
-        origin = typing.get_origin(t)
-        if origin == tuple and typing.get_args(t):
+        origin = typing_get_origin(t)
+        if origin == tuple and typing_get_args(t):
             xs = tuple()
-            for type_arg in typing.get_args(t):
+            for type_arg in typing_get_args(t):
                 xs += extract_types(type_arg)
             return xs
         if (origin == typing.Union
-                or issubclass_safe(origin, abc.Sequence)) and typing.get_args(t):
-            type_arg = typing.get_args(t)[0]
+                or issubclass_safe(origin, abc.Sequence)) and typing_get_args(t):
+            type_arg = typing_get_args(t)[0]
             return extract_types(type_arg)
-        elif issubclass_safe(origin, abc.Mapping) and typing.get_args(t):
-            key_type_arg, value_type_arg = typing.get_args(t)
+        elif issubclass_safe(origin, abc.Mapping) and typing_get_args(t):
+            key_type_arg, value_type_arg = typing_get_args(t)
             if key_type_arg is UUID:
                 return (UUID,) + extract_types(value_type_arg)
             return extract_types(value_type_arg)
